/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package com.aybkamen.tfc.persistencia;

import com.aybkamen.tfc.recursos.RecursoDigital;
import com.aybkamen.tfc.recursos.RecursoTuristico;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntModelSpec;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QuerySolution;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.rdf.model.InfModel;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.reasoner.Reasoner;
import com.hp.hpl.jena.reasoner.ReasonerRegistry;
import com.hp.hpl.jena.util.FileManager;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 *
 * @author aybkamen
 */
public class ConectorDatos {
    
    public static final String BASE_TURISTICOS = "http://www.aybkamen.com/arioch/ontology/RecursosTuristicos.owl";
    public static final String BASE_SPACE = "http://212.119.9.180/Ontologies/0.3/space.owl";
    public static final String BASE_DIGITAL = "http://www.aybkamen.com/arioch/ontology/RecursosDigitales.owl";
    public static final String BASE_MALLORCA = "http://www.aybkamen.com/arioch/ontology/mallorca.owl";
    public static final String BASE_RDF = "http://www.w3.org/1999/02/22-rdf-syntax-ns";
    public static final String BASE_RDFS = "http://www.w3.org/2000/01/rdf-schema";
    public static final String BASE_DC = "http://purl.org/dc/elements/1.1/";

    public static final String QUERY_PREFIX = "PREFIX rdf: <"+BASE_RDF+"#>"
                + " PREFIX rdfs: <"+BASE_RDFS+"#>"
                + " PREFIX mallorca: <"+BASE_MALLORCA+"#> "
                + " PREFIX digital: <"+BASE_DIGITAL+"#> "
                + " PREFIX space: <"+BASE_SPACE+"#> "
                + " PREFIX turistico: <"+BASE_TURISTICOS+"#> "
                + " PREFIX dc: <"+BASE_DC+"#>";

    public static final String PROP_TYPE = BASE_RDF+"#type";
    public static final String PROP_TITLE = BASE_DC+"#title";
    public static final String PROP_MUESTRA = BASE_TURISTICOS+"#muestra";
    public static final String PROP_EstaSituadoEn = BASE_TURISTICOS+"#estaSituadoEn";

    private InfModel model;

    private Model modeloAsertivo;
    private Model modeloInferencia;

    public Model getModeloAsertivo(){
        if(modeloAsertivo==null){
            // Creamos un modelo sin razonador
            modeloAsertivo = ModelFactory.createOntologyModel( OntModelSpec.OWL_MEM);
            // Cargamos los datos desde un fichero RDF/XML
            modeloAsertivo.read(BASE_MALLORCA, "RDF/XML");
        }
        return modeloAsertivo;
    }

    public Model getModeloInferencia(){
        if(modeloInferencia==null){
            // Creamos un modelo con un razonador transitivo
            modeloInferencia = ModelFactory.createOntologyModel(OntModelSpec.OWL_MEM_MICRO_RULE_INF);
            // Cargamos los datos desde un fichero RDF/XML
            modeloInferencia.read(BASE_MALLORCA, "RDF/XML");
        }
        return modeloInferencia;
    }
    
    public InfModel getModel(){
        if(model==null){
            Model schema = FileManager.get().loadModel(BASE_TURISTICOS);
            Model data = FileManager.get().loadModel(BASE_MALLORCA);

            Reasoner r = ReasonerRegistry.getOWLReasoner();
            r = r.bindSchema(schema);
            model = ModelFactory.createInfModel(r,data);
            model.setNsPrefix("rdf", BASE_RDF);
            model.setNsPrefix("rdfs", BASE_RDFS);
            model.setNsPrefix("space", BASE_SPACE);
            model.setNsPrefix("digital", BASE_DIGITAL);
            model.setNsPrefix("turistico", BASE_TURISTICOS);
            model.setNsPrefix("mallorca", BASE_MALLORCA);
        }
        return model;
    }

   public ResultSet select(String queryString, Model modelo) throws Exception{
        queryString = QUERY_PREFIX + queryString;
        QueryExecution qe = QueryExecutionFactory.create(queryString, modelo);
        ResultSet results = qe.execSelect();
        return results;
    }

   public Collection<RecursoDigital> getDigitalesPDI(String pdi){
       List<RecursoDigital> lista = new ArrayList<RecursoDigital>();
       // Preparamos la búsqueda de los digitales del elemento
       String queryString = "SELECT * WHERE  { ?digital turistico:muestra <"+pdi+">}";
       ResultSet results;
        try {
            results = select(queryString, getModeloAsertivo());
            // Para cada recurso encontrado
            while(results.hasNext()){
                QuerySolution r = results.nextSolution();
                RDFNode recurso = r.get("digital");
                // Creamos un RecursoDigital y lo añadimos a la lista
                RecursoDigital rd = new RecursoDigital(recurso.toString(), this);
                lista.add(rd);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
       return lista;
   }

   public RecursoTuristico loadRecursoTuristico(String iri){
       try {
           return new RecursoTuristico(this.BASE_MALLORCA+"#"+iri, this);
       } catch (Exception ex){
           ex.printStackTrace();
       }
       return null;
   }

   public Collection<RecursoTuristico> getRecursos(String query){
       List<RecursoTuristico> lista = new ArrayList<RecursoTuristico>();
       ResultSet results;
       try {
            results = select(query, getModeloInferencia());
            // Para cada recurso encontrado
            while(results.hasNext()){
                QuerySolution r = results.nextSolution();
                RDFNode recurso = r.get("recurso");
                // Creamos un RecursoDigital y lo añadimos a la lista
                RecursoTuristico rt = new RecursoTuristico(recurso.toString(), this);
                lista.add(rt);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
       return lista;
   }

   public Collection<RecursoTuristico> getRecursosPorClase(String idClase){
       List<RecursoTuristico> lista = new ArrayList<RecursoTuristico>();
       String query = "SELECT ?recurso "
               + " WHERE {"
               + "   ?recurso rdf:type "+idClase+"."
               + "}";
       ResultSet results;

       try {
            results = select(query, getModeloInferencia());
            // Para cada recurso encontrado
            while(results.hasNext()){
                QuerySolution r = results.nextSolution();
                RDFNode recurso = r.get("recurso");
                // Creamos un RecursoDigital y lo añadimos a la lista
                RecursoTuristico rt = new RecursoTuristico(recurso.toString(), this);
                lista.add(rt);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
       return lista;
   }

   public Collection<RecursoTuristico> getRecursosPorLugar(String idLugar){
       List<RecursoTuristico> lista = new ArrayList<RecursoTuristico>();
       String query = "SELECT ?recurso "
               + " WHERE {"
               + "   ?recurso turistico:estaSituadoEn "+idLugar+"."
               + "}";
       ResultSet results;

       try {
            results = select(query, getModeloInferencia());
            // Para cada recurso encontrado
            while(results.hasNext()){
                QuerySolution r = results.nextSolution();
                RDFNode recurso = r.get("recurso");
                // Creamos un RecursoDigital y lo añadimos a la lista
                RecursoTuristico rt = new RecursoTuristico(recurso.toString(), this);
                lista.add(rt);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
       return lista;
   }
}
